# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-04-24 14:24
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Answer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField()),
                ('last_update', models.DateField(auto_now=True, null=True)),
                ('author', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name_plural': 'Ответы',
                'verbose_name': 'Ответ',
            },
        ),
        migrations.CreateModel(
            name='Question',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('target_id', models.PositiveIntegerField()),
                ('title', models.CharField(blank=True, max_length=200, null=True, unique=True)),
                ('text', models.TextField()),
                ('last_update', models.DateField(auto_now=True, null=True)),
                ('author', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
                ('target_content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
            options={
                'verbose_name_plural': 'Вопросы',
                'verbose_name': 'Вопрос',
            },
        ),
        migrations.AddField(
            model_name='answer',
            name='question',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='qna.Question'),
        ),
        migrations.AlterUniqueTogether(
            name='answer',
            unique_together=set([('author', 'question')]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-08-02 11:19
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('screenshot', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='screenshot',
            name='height',
            field=models.IntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='screenshot',
            name='width',
            field=models.IntegerField(blank=True, null=True),
        ),
    ]



from django.contrib import  auth
from .translitirate import transliterate
import re
from django.contrib.contenttypes.models import ContentType
import bleach
import re


class CommonHelpers:
    @staticmethod
    def get_user__by_request(request):
        if not hasattr(request, '_cached_user'):
            request._cached_user = auth.get_user(request)
        return request._cached_user

    @staticmethod
    def get_slug(_str):
        _str = _str.strip()
        slug = _str

        slug = transliterate(slug)
        slug = slug.lower()
        slug = re.sub(r'[^a-z0-9_-]', r'', slug)

        return slug



# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-04-14 14:35
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('review', '0003_auto_20160413_0517'),
    ]

    operations = [
        migrations.AlterField(
            model_name='review',
            name='cons',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='review',
            name='pros',
            field=models.TextField(blank=True, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-04-30 18:56
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('qna', '0009_auto_20160430_1521'),
    ]

    operations = [
        migrations.AddField(
            model_name='answer',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-04-27 09:59
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('review', '0006_auto_20160425_1011'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='review',
            options={'verbose_name': 'Отзыв', 'verbose_name_plural': 'Отзывы'},
        ),
        migrations.AlterUniqueTogether(
            name='review',
            unique_together=set([('target_content_type', 'target_id', 'author')]),
        ),
    ]



from django import forms
from django.contrib.auth.models import User
from .models import Review
from django.contrib.contenttypes.fields import GenericForeignKey
from django.utils.translation import ugettext as _


class ReviewForm(forms.ModelForm):

    # def save(self):
    #     print('save')
    #     data = super(ReviewForm, self).clean()
    #     print('data', data)
    #     # data['author'] = data['author'].id
    #     print(self.Meta.model)
    #     # data['target_id'] = data['target'].id
    #     new_review = self.Meta.model(data)
    #     print('new_review ', new_review )
    #     new_review.save()

        # print(self)

    # def clean(self):
    #     cleaned_data = super(ReviewForm, self).clean()
    #     print('cleaned_data', cleaned_data)

    # def clean_rating(self):
    #     rating = self.cleaned_data['rating']
    #     print('clean rating -', rating, 0 < rating <= 5)
    #     if not 0 < rating <= 5:
    #         raise forms.ValidationError(_("This field is required."))
    #
    #     return rating

    def clean_rating(self):
        rating = self.data['rating']

        try:
            rating = int(rating)

            if not rating > 0:
                raise forms.ValidationError(_("This field is required."))
        except:
            raise forms.ValidationError(_("This field is required."))

        return rating

    def clean_advise(self):
        advise = self.data['advise']

        if advise == 'true':
            advise = True
        elif advise == 'false':
            advise = False

        # print('advise', advise)

        if not (advise == True or advise == False):
            raise forms.ValidationError(_("This field is required."))

        return advise

    class Meta:
        model = Review
        fields = ['pros', 'cons', 'summary', 'advise',
                  'rating', 'author', 'author_anonym', 'target_id', 'target_content_type']




from .models import LikeStats, Like
from gfk_models.gfk_base.mixins import GFKItemsStatsManager, GFKBaseMixin
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.contenttypes.fields import GenericRelation


class LikeStatsManager(GFKItemsStatsManager):
    stats_model = LikeStats
    item_model = Like


class LikesMixin(GFKBaseMixin):
    likes = GenericRelation(Like,
                            # related_query_name='target',
                            object_id_field='target_id',
                            content_type_field='target_content_type')

    likes_stats = GenericRelation(LikeStats,
                            # related_query_name='target',
                            object_id_field='target_id',
                            content_type_field='target_content_type')

    def likes_manager(self):
        return LikeStatsManager(self)

    def __add_rating(self, user, value):
        # print('__add_rating', self, user, value)
        try:
            likes = self.likes_manager().stats()
            like = Like.objects.get(_target=self, author=user)
            likes.sum += -like.rating + value
            like.rating = value

        except ObjectDoesNotExist:
            like = Like(target=self, rating=value, author=user)
            likes.sum += value
            likes.count += 1

        like.save()
        # If there is no Exceptions then it's ok
        return likes

    def like(self, user):
        return self.__add_rating(user, +1)

    def dislike(self, user):
        return self.__add_rating(user, -1)

    class Meta:
        abstract = True


# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-04-27 14:22
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('qna', '0005_auto_20160426_0908'),
    ]

    operations = [
        migrations.AddField(
            model_name='answer',
            name='accepted',
            field=models.BooleanField(default=False),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-04-30 15:21
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('qna', '0008_auto_20160430_1304'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='question',
            options={'ordering': ['created_at'], 'verbose_name': 'Вопрос', 'verbose_name_plural': 'Вопросы'},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-04-30 13:04
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('qna', '0007_auto_20160429_1128'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='question',
            options={'ordering': ['-last_update'], 'verbose_name': 'Вопрос', 'verbose_name_plural': 'Вопросы'},
        ),
        migrations.AddField(
            model_name='question',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-04-24 08:27
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('review', '0004_auto_20160414_1435'),
    ]

    operations = [
        migrations.AddField(
            model_name='review',
            name='last_update',
            field=models.DateField(auto_now=True, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-08-03 12:43
from __future__ import unicode_literals

from django.db import migrations
import sorl.thumbnail.fields


class Migration(migrations.Migration):

    dependencies = [
        ('screenshot', '0002_auto_20160802_1119'),
    ]

    operations = [
        migrations.AlterField(
            model_name='screenshot',
            name='src',
            field=sorl.thumbnail.fields.ImageField(height_field='height', upload_to='screenshots/', width_field='width'),
        ),
    ]



from django.contrib.auth.decorators import login_required
from django.utils.decorators import method_decorator
from django.shortcuts import redirect
from django.http.response import JsonResponse
from django.forms import ValidationError
from django.contrib.auth.mixins import LoginRequiredMixin
from django.views import generic

from gfk_models.gfk_base.views import GFKListView, GFKDetailView, GFKCreateView, GFKUpdateView
from gfk_models.gfk_helpers.functions.common import CommonHelpers
from .forms import QuestionForm, AnswerForm
from .models import Question, Answer
from hitcount.views import HitCountDetailView


class QuestionCreateView(LoginRequiredMixin, GFKCreateView):
    model = Question
    form_class = QuestionForm
    template_name = "qna/add.html"

    def __init__(self, *args, **kwargs):
        # self.login_url = reverse('auth:sign_in')
        return super().__init__(*args, **kwargs)

    @method_decorator(login_required)
    def post(self, request, *args, **kwargs):

        post_dict = request.POST.copy()
        post_dict.update({'author': CommonHelpers.get_user__by_request(request).id})
        request.POST = post_dict

        return super(QuestionCreateView, self).post(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        data = super().get_context_data(**kwargs)
        data['submit_button_text'] = "Задать вопрос"
        return data


class QuestionUpdateView(LoginRequiredMixin, generic.UpdateView):
    model = Question
    form_class = QuestionForm
    pk_url_kwarg = 'q_id'

    def __init__(self, *args, **kwargs):
        # self.login_url = reverse('auth:sign_in')
        return super().__init__(*args, **kwargs)

    template_name = "qna/add.html"

    def dispatch(self, request, *args, **kwargs):

        return super().dispatch(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):

        post_dict = request.POST.copy()
        post_dict.update({'author': CommonHelpers.get_user__by_request(request).id})
        request.POST = post_dict

        return super(QuestionUpdateView, self).post(request, *args, **kwargs)

    def get_object(self, queryset=None):
        object = super().get_object(queryset)

        if object.author.id != self.request.user.id:
            raise ValidationError('Редактировать можно только свои вопросы и ответы.')

        return object

    def get_context_data(self, **kwargs):
        data = super().get_context_data(**kwargs)
        data['submit_button_text'] = "Сохранить вопрос"
        return data


class QuestionListView(GFKListView):
    model = Question
    paginate_by = 10
    paginate_orphans = 5

    template_name = "qna/list.html"

    def get_context_data(self, **kwargs):
        data = super().get_context_data(**kwargs)
        data['sidebar_popular'] = Question.get_popular_today()

        return data


class QuestionDetailView(HitCountDetailView, GFKDetailView):
    model = Question
    count_hit = True
    pk_url_kwarg = 'q_id'

    template_name = "qna/single.html"

    def get_target(self):
        return

    def get_context_data(self, **kwargs):
        data = super().get_context_data(**kwargs)
        data['target'] = data['object'].get_target()
        data['sidebar_popular'] = Question.get_popular_today()

        return data

    def get(self, request, *args, **kwargs):
        question = Question.objects.get(id=kwargs['q_id'])

        if question.get_slug() != kwargs['q_slug']:
            return redirect(question.get_absolute_url())
        else:
            return super().get(request, *args, **kwargs)


class QuestionAcceptAnswerView(LoginRequiredMixin, generic.DetailView):
    model = Question
    pk_url_kwarg = 'q_id'

    def __init__(self, *args, **kwargs):
        # self.login_url = reverse('auth:sign_in')
        return super().__init__(*args, **kwargs)

    @method_decorator(login_required)
    def get(self, request, *args, **kwargs):
        question = Question.objects.get(id=kwargs['q_id'])
        result = question.toggle_accepted_answer(kwargs['a_id'], request.user)

        return JsonResponse({
            'checked': result
        })


class AnswerCreateView(LoginRequiredMixin, generic.CreateView):
    model = Answer
    form_class = AnswerForm

    template_name = "qna/answer_add.html"

    def __init__(self, *args, **kwargs):
        # self.login_url = reverse('auth:sign_in')
        return super().__init__(*args, **kwargs)

    def post(self, request, *args, **kwargs):
        post_dict = request.POST.copy()
        post_dict.update({'author': CommonHelpers.get_user__by_request(request).id})
        request.POST = post_dict

        return super().post(request, *args, **kwargs)


class AnswerUpdateView(LoginRequiredMixin, generic.UpdateView):
    model = Answer
    pk_url_kwarg = 'a_id'
    form_class = AnswerForm

    template_name = "qna/answer_add.html"

    def __init__(self, *args, **kwargs):
        # self.login_url = reverse('auth:sign_in')
        return super().__init__(*args, **kwargs)

    def post(self, request, *args, **kwargs):
        post_dict = request.POST.copy()
        post_dict.update({'author': CommonHelpers.get_user__by_request(request).id})
        request.POST = post_dict

        return super().post(request, *args, **kwargs)

    def get_object(self, queryset=None):
        object = super().get_object(queryset)

        if object.author.id != self.request.user.id:
            raise ValidationError('Редактировать можно только свои вопросы и ответы.')

        return object



from django.contrib import auth
from .translitirate import transliterate

from django.contrib.contenttypes.models import ContentType
import bleach
import re


class TextProcessors:
    text_allowed_tags = ['a', 'img', 'div', 'p', 'h2', 'h3', 'h4', 'em', 'b', 'strong', 'ul', 'ol', 'li', 'blockquote']
    text_allowed_attrs = ['alt', 'rel', 'href']

    @classmethod
    def text_sanitize(cls, text, text_allowed_tags=None, text_allowed_attrs=None):

        if text_allowed_tags is None:
            text_allowed_tags = cls.text_allowed_tags

        if text_allowed_attrs is None:
            text_allowed_attrs = cls.text_allowed_attrs

        text = bleach.clean(text,
                            tags=text_allowed_tags,
                            attributes=text_allowed_attrs,
                            strip=True
                        )
        text = cls.__clean_blank_tags(text, text_allowed_tags)
        text = cls.__clean_line_breaks(text)
        callbacks = [
            cls.__add__a_rel_nofollow,
            cls.__add__a_target,
            cls.__add__a_href_protocol
        ]
        text = bleach.linkify(text, callbacks=callbacks)

        return text

    @staticmethod
    def __clean_blank_tags(text, tags):
        for tag in tags:
            pattern = "<%s>\s</%s>" % (tag, tag)
            # print(pattern)
            regex = re.compile(pattern, re.IGNORECASE)
            text = regex.sub('', text)

        return text

    @staticmethod
    def __clean_line_breaks(text):
        pattern = "\r"
        regex = re.compile(pattern, re.IGNORECASE)
        text = regex.sub('', text)

        pattern = "\n"
        regex = re.compile(pattern, re.IGNORECASE)
        text = regex.sub('', text)

        return text

    @staticmethod
    def __add__a_rel_nofollow(attrs, new=False):
        attrs['rel'] = 'nofollow'
        return attrs

    @staticmethod
    def __add__a_href_protocol(attrs, new=False):
        # print(attrs)
        # if not attrs['href'].startswith("http://") and \
        #    not attrs['href'].startswith("https://"):
        #     attrs['href'] = "http://%s" % attrs['href']

        return attrs

    @staticmethod
    def __add__a_target(attrs, new=False):
        attrs['target'] = '_blank'
        return attrs

    @staticmethod
    def text_linkify(text):
        return bleach.linkify(text)

    @staticmethod
    def get_content_type(target):
        return ContentType.objects.get_for_model(target)

    @staticmethod
    def get_content_type_by_id(ct):
        return ContentType.objects.get(id=ct)

    @classmethod
    def get_target(cls, target_ct, target_id):
        # print('get_target', target_ct, target_id)
        return cls.get_content_type_by_id(target_ct).model_class().objects.get(id=target_id)

    @staticmethod
    def get_attr_list(arr, attr_name):
        '''
        :param arr: list of objects
        :param attr_name: object attribute
        :return: new list of 'arr' item object 'attr_name' attributes
        '''
        attr_arr = []
        for elem in arr:
            attr_arr.append(getattr(elem, attr_name))

        return attr_arr

    @staticmethod
    def list_sort__by_another_list_items_attr(list_to_sort, attr_name, list_sorted_by):
        '''
        Сортировка списка объектов по списку параметров этих объектов,
        например сортровка [Taxonomy] по списку id этих Taxonomy.

        Выдает ошибку если хотя бы в одном элементе 'list_to_sort' нет атрибута 'attr_name'
        :param list_to_sort:
        :param attr_name:
        :param list_sorted_by:
        :return: [typeof(list_to_sort[0]]
        '''
        list_sorted = []
        for item in list_sorted_by:
            try:
                list_sorted.append([s_item for s_item in list_to_sort if getattr(s_item, attr_name) == item][0])
            except:
                pass

        return list_sorted


import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='gfk-models',
    version='0.0.13',
    packages=[
        'gfk_models',
        'gfk_models.gfk_base',
        'gfk_models.gfk_base.templatetags',
        'gfk_models.gfk_helpers',
        'gfk_models.gfk_helpers.functions',
        'gfk_models.vote',

        'gfk_models.like',
        'gfk_models.like.migrations',

        'gfk_models.qna',
        'gfk_models.qna.migrations',

        'gfk_models.review',
        'gfk_models.review.migrations',

        'gfk_models.screenshot',
        'gfk_models.screenshot.migrations',
    ],
    include_package_data=True,
    license='BSD License',  # example license
    description='Set of GenericForeignKey models.',
    # long_description=README,
    # long_description_content_type="text/markdown",
    # variant="CommonMark",
    author='Alexander Temchenko',
    author_email='ksanderer@gmail.com',
    install_requires=[
        "django >=2.2",
        'bleach >=1.4.2'
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.9',  # replace "X.Y" as appropriate
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        # Replace these appropriately if you are stuck on Python 2.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)


from django.db import models
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ObjectDoesNotExist


class GFKQuerySet(models.QuerySet):

    def filter(self, *args, **kwargs):
        if '_target' in kwargs:
            kwargs['target_id'] = kwargs['_target'].id
            kwargs['target_content_type'] = ContentType.objects.get_for_model(kwargs['_target']).id
            kwargs.pop("_target", None)

        # print('GFKQuerySet -> filter', self, args, kwargs)

        return super(GFKQuerySet, self).filter(*args, **kwargs)

    def get(self, *args, **kwargs):
        if '_target' in kwargs:
            kwargs['target_id'] = kwargs['_target'].id
            kwargs['target_content_type'] = ContentType.objects.get_for_model(kwargs['_target']).id
            kwargs.pop("_target", None)

        # print('GFKQuerySet -> get', self, args, kwargs)

        return super(GFKQuerySet, self).get(*args, **kwargs)


class GFKManger(models.Manager):
    def get_queryset(self):
        return GFKQuerySet(self.model, using=self._db)

    def filter(self, *args, **kwargs):
        if '_target' in kwargs:
            kwargs['target_id'] = kwargs['_target'].id,
            kwargs['target_content_type'] = ContentType.objects.get_for_model(kwargs['_target']).id
            kwargs.pop("_target", None)

        # print('GFKManger -> filter',self, args, kwargs)
        new_qs = super(GFKManger, self).get_queryset().filter(*args, **kwargs)
        new_gfk_qs = new_qs._clone()
        return super(GFKManger, self).get_queryset().filter(*args, **kwargs)

    def get(self, *args, **kwargs):
        if '_target' in kwargs:
            kwargs['target_id'] = kwargs['_target'].id
            kwargs['target_content_type'] = ContentType.objects.get_for_model(kwargs['_target']).id
            kwargs.pop("_target", None)

        # print(self, args, kwargs)

        return super(GFKManger, self).get_queryset().get(*args, **kwargs)


class GFKModel(models.Model):
    target_content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    target_id = models.PositiveIntegerField()
    target = GenericForeignKey('target_content_type', 'target_id')
    # placeholder для GFKManger.filter чтобы можно было получать данные по target автоматом
    # нужно использовать objects.filter(_target=target), в GFKManager _target будет
    # преобразован в target_content_type и target_id
    _target = models.BooleanField(default=True, blank=True)

    objects = GFKQuerySet.as_manager() # GFKManger()

    class Meta:
        abstract = True

    def get_target(self):
        # print('self.target_id', self.target_id)
        # content_type = ContentType.objects.get(id=self.target_content_type)
        return self.target_content_type.model_class().objects.get(id=self.target_id)

    @staticmethod
    def get_target_static(target):
        content_type = ContentType.objects.get(id=target.target_content_type)
        return content_type.model_class().objects.get(id=target.target_id)

    @staticmethod
    def get_ct_model_static(target_ct_id):
        return ContentType.objects.get(id=target_ct_id).model_class()


    @staticmethod
    def get_content_type(target):
        return ContentType.objects.get_for_model(target).id

    @classmethod
    def filter_by_target(cls, target):

        return cls.objects.filter(
            target_id=target.id,
            target_content_type=ContentType.objects.get_for_model(target).id
        )


from django.db.models import signals


class GFKStats(GFKModel):
    # Model to count stats
    item_model = None

    recount_attr_list = ['count']

    count = models.IntegerField(default=0)

    class Meta:
        unique_together = ['target_content_type', 'target_id']
        abstract = True

    def recount_count(self):
        return self.get_recount_queryset().count()

    def recount_stats(self):
        '''
        This method invokes by signal each time when model instance has been updated
        :return:
        '''
        for attr in self.recount_attr_list:
            recount_attr = getattr(self, 'recount_%s' % attr)
            setattr(self, attr, recount_attr())

        self.save()

    def get_recount_queryset(self):
        # We count statistics for same target as GFKStats target
        kwargs = {
            'target_content_type': self.target_content_type,
            'target_id': self.target_id
        }
        return self.item_model.objects.filter(**kwargs)

    @classmethod
    def setup_relation(cls, model):
        '''
        Set up relation with GFK model which statics we should count.
        '''
        cls.item_model = model
        signals.post_save.connect(cls.signal_callback, sender=model)

    @classmethod
    def signal_callback(cls, **kwargs):
        inst = kwargs['instance']

        kwargs = {
            'target_content_type': inst.target_content_type,
            'target_id': inst.target_id
        }

        try:
            stats_object = cls.objects.get(**kwargs)
        except ObjectDoesNotExist:
            stats_object = cls(**kwargs)
            stats_object.save()

        stats_object.recount_stats()


from django.contrib import admin
from .models import Screenshot


class ScreenshotAdmin(admin.ModelAdmin):
    model = Screenshot

admin.site.register(Screenshot, ScreenshotAdmin)


# class ReviewListAdmin(admin.ModelAdmin):
#     model = ReviewList

# admin.site.register(ReviewList, ReviewListAdmin)
